/* webget.p
 *
 * simple program for testing socket communications code
 *
 * [b]pro -p util/webget.p -param "demo.wss.com|80|/ping"
 *
 * webget.log is created or overwritten with each invocation
 *
 * some other, possibly interesting, files to test with:
 *
 *	/proxy.test			small text file: "test passed"
 *	/pt3upd/messages.cfg		medium sized, plain text, 64k ish
 *	/pt3upd/protop-323-rc2.zip	large, binary file
 *
 * parameters:
 *
 *	host		host to communicate with
 *	port		port number
 *	path		path to the resource for GET method
 *	proxy		optional: proxy string
 *	credentials	optional: base64 encoded username:password
 *	options		optional: CONNECT() i.e. -ssl -nohostverify
 *	file name	optional: if specified the body of the request (but not the headers) will be saved to this file
 *	extra headers	optional: additional request headers
 *	debug mode	optional: default is 5, 7 is about as high as you want to go
 *
 * environment variables that might have an impact:
 *
 *	PTHOST		default host
 *	PORTALPORT	default port
 *	PORTALOPTIONS	default CONNECT() options
 *	PROXY		default proxy string
 *	HTTPVERSION	default HTTP/1.0, optional: HTTP/1.1
 *	WAITLIMIT	default timeout for socket connections, 
 *	USESOCKETS	yes disables all socket communications
 */

define new global shared variable dbgMode    as integer   no-undo.

define variable host     as character no-undo format "x(40)" initial "localhost".
define variable port     as character no-undo format "x(40)" initial "80".
define variable path     as character no-undo format "x(40)".
define variable proxy    as character no-undo format "x(250)" view-as fill-in size 40 by 1 label "Optional Proxy".
define variable creds    as character no-undo format "x(40)"   initial "" label "Encoded Credentials".
define variable options  as character no-undo format "x(256)"  initial "" view-as editor size 40 by 1 label "CONNECT() Options".
define variable fName    as character no-undo format "x(40)" label "Output File Name".
define variable xtraHdrs as character no-undo format "x(4096)" initial "" view-as editor size 40 by 1 label "Extra Headers".

define variable result   as longchar  no-undo.

define variable rLen     as integer   no-undo format ">,>>>,>>>,>>9" label "result length".
define variable fLen     as integer   no-undo format ">,>>>,>>>,>>9" label "file length".
define variable xstatus  as character no-undo format "x(20)".

define variable xrspHdr  as character no-undo view-as editor size 70 by 3.
define variable xresult  as character no-undo view-as editor size 70 by 3.

define variable startTime  as datetime-tz no-undo.
define variable runTime    as decimal     no-undo.
define variable throughPut as decimal     no-undo format ">,>>>,>>9.999 Mbps".

/* create a new log at each session startup & capture socket library configuration
 */

dbgMode = 5.	/* 7 is about as high as you really want to go */

if opsys = "unix" then
  input from value( "/dev/null" ).
 else
  input from value( "NUL" ).

output to value( "webget.log" ).
run ssg/sausage02.p persistent.
output close.

input close.

host = os-getenv( "PTHOST" ).
if host = ? then host = "".

port = os-getenv( "PORTALPORT" ).
if port = ? then port = "".

options = os-getenv( "PORTALOPTIONS" ).
if options = ? then options = "".

proxy = os-getenv( "PROXY" ).
if proxy = ? then proxy = "".

/* override the default environment if desired
 */

if num-entries( session:parameter, "|" ) >= 1 then host     = entry( 1, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 2 then port     = entry( 2, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 3 then path     = entry( 3, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 4 then proxy    = entry( 4, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 5 then creds    = entry( 5, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 6 then options  = entry( 6, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 7 then fName    = entry( 7, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 8 then xtraHdrs = entry( 8, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 9 then dbgMode  = integer( entry( 9, session:parameter, "|" )).

if host = "" then host = "demo.wss.com".
if port = "" then port = "80".
if path = "" then path = "/ping".

if session:batch = no and fname = "" then
  fname = "webget.result".

pause 0 before-hide.

do while true on endkey undo, leave:

  if session:batch = no then
    do:
      update
        dbgMode label "debug level" colon 24 skip
        host     colon 24 skip
        port     colon 24 skip
        path     colon 24 skip
        proxy    colon 24 skip
        creds    colon 24 skip
        options  colon 24 "..." skip
        fName    colon 24 skip
        xtraHdrs colon 24 "..." skip
       with
        side-labels
        no-box
        width 80
      .
    end.

  output to value( "webget.log" ) append.

  message now "====" "BEGIN" fill( "=", 80 ).

  message now substitute( "host: &1, port: &2, path: &3, proxy: &4, credentials: &5, options: &6, file name: &7, dbgMode: &8", host, port, path, proxy, creds, options, fname, dbgMode ).
  message now "extra headers:" xtraHdrs.

  startTime = now.

  result = "".
  fix-codepage( result ) = 'utf-8'.

  run getURL( host, port, path, creds, options, proxy, xtraHdrs, fName, output result, output xrspHdr ).

  runTime = interval( now, startTime, "milliseconds" ) / 1000.0.

  xstatus = entry( 1, xrspHdr, "~n" ).

  xresult = substring( result, 1, 4096 ).

  rLen = length( result ).

  if fName <> "" and fName <> ? then
    do:
      file-info:file-name = fName.
      fLen = file-info:file-size.
    end.

  throughPut = ( fLen * 8.0 / ( 1024 * 1024 )) / runTime.

  message now "runtime: " runTime " throughput:" throughPut "MB/sec".

  message now "====" "END".
  message skip.

  output close.

  if session:batch = yes then
    do:
      quit.
    end.
   else
    do:
      display
        skip(1)
        xStatus colon 16 throughPut colon 60 skip
        rLen    colon 16 fLen       colon 60 skip
        skip(1)
        xResult skip
        skip(1)
        xrspHdr skip
      .
    end.
end.

quit.
